@@ -6,7 +6,7 @@ from .P2SH       import P2SH
 from .P2WPKH     import P2WPKH
 from .bip39      import Bip39MnemonicGenerator, Bip39MnemonicValidator, Bip39SeedGenerator
 from .bip32      import Bip32Const, Bip32, PathParser
-from .bip44_base import Bip44Chains, Bip44Coins
+from .bip44_base import Bip44Changes, Bip44Coins
 from .bip44      import Bip44
 from .bip49      import Bip49
 from .bip84      import Bip84
